#!/usr/bin/env python
# -*- coding: cp1252 -*-
# This code is part of DPA Contest V4 project
# Copyright 2013 Telecom ParisTech

import sys
import os
import threading
import time
import logging
import math
import serial

class SmartcardSerial():
    
    def __init__(self):
        self.ser = None

    def __del__(self):
        self.close()
    
    def connect(self, serport):
        if self.ser == None:
            # Open serial port 
            self.ser = serial.Serial()
            self.ser.port     = serport
            self.ser.baudrate = 9600
            self.ser.timeout  = 3     # 3 second timeout
            self.ser.stopbits = serial.STOPBITS_TWO
            self.ser.parity = serial.PARITY_EVEN
            self.ser.open()
        self.reset()

    def reset(self):
        self.ser.setRTS(True)
        time.sleep(0.2)
        self.ser.setRTS(False)
        time.sleep(0.5)

        data = bytearray(self.ser.read(self.ser.inWaiting()))
        stratr = "ATR = "
        for p in data:
            stratr = stratr + "%2X "%p

        print stratr
        return stratr
    
    def disconnect(self):
        self.close()
            
    def close(self):
        if self.ser != None:
            self.ser.close()
            self.ser = None
        
    def init(self):
        self.reset()

    def getATR(self):
        return self.stratr

    def APDUHeader(self, cla, ins, p1, p2, datalen):
        header = bytearray()
        header.append(cla)
        header.append(ins)
        header.append(p1)
        header.append(p2)
        header.append(datalen)
        header=str(header)
        self.ser.flushInput()
        self.ser.write(header)
        
        try:
            resp = bytearray(self.ser.read(2))
            if (resp[1] != ins):
                print "SASEBOW: APDUHeader Sync Error %X != %X"%(resp[1], ins)
                return False
        except:
            print "SASEBOW: APDUHeaderSync Error, no response"
            return False

        return True

    def APDUSend(self, cla, ins, p1, p2, data):
        self.APDUHeader(cla, ins, p1, p2, len(data))   
        self.ser.write(str(bytearray(data)))
        self.ser.read(len(data))
        resp = self.ser.read(2)

        if len(resp) < 2:
            print "SASEBOW: APDUSend Error, no response"

        return True

    def APDURecv(self, cla, ins, p1, p2, datalen):
        self.APDUHeader(cla, ins, p1, p2, datalen)
        resp = self.ser.read(datalen + 2)

        if len(resp) < (datalen + 2):
            print "SASEBOW: APDUResp Error, response too short"
            return 0

        resp = bytearray(resp)
        
        return resp[0:datalen]        
                    
    def enc(self,maxr):
        data_tab = open("./pt_samples", "r")
        for i in range(0,maxr):
            data=data_tab.readline()[:-1]
            data=data.decode('hex')
            self.APDUSend(0x80,0xC0,0x00,0x00,data)
            time.sleep(0.5) 
            data = bytearray(self.ser.read(self.ser.inWaiting()))
            buf = "data : "
            for p in data:
                buf = buf + "%2X "%p
            print buf
            self.ser.flush()


def __main__():
    time.sleep(1)
    test=SmartcardSerial()
    test.connect("/dev/ttyUSB0")
    time.sleep(0.5)
    test.enc(50)

__main__()

