#include "config.h"

.arch	ARCH
#define _SFR_ASM_COMPAT 1 
#define __SFR_OFFSET 0 
#include <avr/io.h> 

#include "avr-asm-macros.S"

shuffled_aes_rsm_begin:

ST11=0
ST21=1
ST31=2
ST41=3
ST12=4
ST22=5
ST32=6
ST42=7
ST13=8
ST23=9
ST33=10
ST43=11
ST14=12
ST24=13
ST34=14
ST44=15

H1=16
H2=17
H3=18
H4=19

I3=20
I=21 
I2=23

S0=24
S1=25

offset=22
offset_base=20

.equ __plain__		, 0x0240
.equ __key__		, 0x0250
.equ __header__		, 0x0260
.equ __shuffle__	, 0x0280
.equ __shuffle10__	, 0x0280
.equ __shuffle0__	, 0x0230
.equ __offset__		, 0x0300
.equ __reg_out__	, 0x0310

.global aes_rsm_core_cenc

aes_rsm_core_cenc:
		push_range 0, 31
		clr r0
		clr r1
		clr r2
		clr r3
		clr r4
		clr r5
		clr r6
		clr r7
		clr r8
		clr r9
		clr r10
		clr r11
		clr r12
		clr r13
		clr r14
		clr r14
		clr r15
		clr r16
		clr r16
		clr r17
		clr r18
main:	cli			; initialize stack
	ldi ZH, 0x02	; loading key into registers
	ldi ZL, 0x50
	ld ST11, Z+
	ld ST21, Z+
	ld ST31, Z+
	ld ST41, Z+
	ld ST12, Z+
	ld ST22, Z+
	ld ST32, Z+
	ld ST42, Z+
	ld ST13, Z+
	ld ST23, Z+
	ld ST33, Z+
	ld ST43, Z+
	ld ST14, Z+
	ld ST24, Z+
	ld ST34, Z+
	ld ST44, Z+

	load_mctable:
	ldi ZH, hi8(__reg_out__)
	ldi ZL, lo8(__reg_out__)
	ldi H1,0
	std Z+0, H1
	ldi H1, 13
	std Z+1, H1
	ldi H1,10
	std Z+2, H1
	ldi H1, 7
	std Z+3, H1
	ldi H1,4
	std Z+4, H1
	ldi H1,1
	std Z+5, H1
	ldi H1, 14
	std Z+6, H1
	ldi H1, 11
	std Z+7, H1
	ldi H1, 8
	std Z+8, H1
	ldi H1, 5
	std Z+9, H1
	ldi H1, 2
	std Z+10, H1
	ldi H1, 15
	std Z+11, H1
	ldi H1, 12
	std Z+12, H1
	ldi H1, 9
	std Z+13, H1
	ldi H1, 6
	std Z+14, H1
	ldi H1, 3
	std Z+15, H1

	ldi XH, 0x01		; expand key to the memory
	ldi XL, 0x10		; locations 0x60..0x60+(16*11-1)
	rcall key_expand
	
	ldi ZH, hi8(m)	; load mask into ST11-ST44 registers
	ldi ZL, lo8(m)
	lpm ST11, Z+
	lpm ST21, Z+
	lpm ST31, Z+
	lpm ST41, Z+
	lpm ST12, Z+
	lpm ST22, Z+
	lpm ST32, Z+
	lpm ST42, Z+
	lpm ST13, Z+
	lpm ST23, Z+
	lpm ST33, Z+
	lpm ST43, Z+
	lpm ST14, Z+
	lpm ST24, Z+
	lpm ST34, Z+
	lpm ST44, Z+

	ldi YH, 0x02		; set the ram pointer next the key block 
	ldi YL, 0x00		; 

	cpmask:			    ; Stores the mask into the ram 
	st Y+, ST11
	st Y+, ST21
	st Y+, ST31
	st Y+, ST41
	st Y+, ST12
	st Y+, ST22
	st Y+, ST32
	st Y+, ST42
	st Y+, ST13
	st Y+, ST23
	st Y+, ST33
	st Y+, ST43
	st Y+, ST14
	st Y+, ST24
	st Y+, ST34
	st Y+, ST44

	; now the registers ST11-ST44 contain the plaintext
	
	
	rcall encrypt		; encryption round main routine


main0:	;stop
	ldi r30 , 0x40
	ldi r31 , 0x02
	ldi r29 , 0x00
	ldi r28 , 0x00

	save_data:
	ld  r16 , Y+
	st  Z+ , r16
	cpi r28, 0x10
	brne save_data

	pop_range 0, 31
	ret


;;; ***************************************************************************
;;; 
;;; KEY_EXPAND
;;; The following routine implements the Rijndael key expansion algorithm. The 
;;; caller supplies the 128 bit key in the registers ST11-ST44 and a pointer 
;;; in the YH:YL register pair. The key is expanded to the memory 
;;; positions [Y : Y+16*11-1]. Note: the key expansion is necessary for both
;;; encryption and decryption.
;;; 
;;; Parameters:
;;;     ST11-ST44:	the 128 bit key
;;;         YH:YL:	pointer to ram location
;;; Touched registers:
;;;     ST11-ST44,H1-H3,ZH,ZL,YH,YL

key_expand:
	ldi H1, 1
	ldi H2, 0x1b
	ldi ZH, hi8(sbox)
	rjmp keyexp1
keyexp0:mov ZL, ST24
	lpm H3, Z
	eor ST11, H3
	eor ST11, H1
	mov ZL, ST34
	lpm H3, Z
	eor ST21, H3
	mov ZL, ST44
	lpm H3, Z
	eor ST31, H3
	mov ZL, ST14
	lpm H3, Z
	eor ST41, H3
	eor ST12, ST11
	eor ST22, ST21
	eor ST32, ST31
	eor ST42, ST41
	eor ST13, ST12
	eor ST23, ST22
	eor ST33, ST32
	eor ST43, ST42
	eor ST14, ST13
	eor ST24, ST23
	eor ST34, ST33
	eor ST44, ST43
	lsl H1
	brcc keyexp1
	eor H1, H2
keyexp1:
	st X+, ST11
	st X+, ST21
	st X+, ST31
	st X+, ST41
	st X+, ST12
	st X+, ST22
	st X+, ST32
	st X+, ST42
	st X+, ST13
	st X+, ST23
	st X+, ST33
	st X+, ST43
	st X+, ST14
	st X+, ST24
	st X+, ST34
	st X+, ST44
	cpi H1, 0x6c
	brne keyexp0
	ret


;;; ***************************************************************************
;;; 
;;; ENCRYPTE 
;;; This routine encrypts a 128 bit plaintext block (supplied in ST11-ST44), 
;;; using an expanded key given in YH:YL. The resulting 128 bit ciphertext
;;; block is stored in ST11-ST44.
;;;
;;; Parameters:
;;;         YH:YL:	pointer to expanded key
;;;         ST11-ST44:  128 bit plaintext block
;;; Touched registers:
;;;     ST11-ST44,H1,H2,H3,I,ZH,ZL,YH,YL
;;; Clock cycles:	11436
		
encrypt:
	ldi I, 10
	clr I2

	clr ST11
	clr ST21
	clr ST31
	clr ST41
	clr ST12
	clr ST22
	clr ST32
	clr ST42
	clr ST13
	clr ST23
	clr ST33
	clr ST43
	clr ST14
	clr ST24
	clr ST34
	clr ST44

	ldi S1, 0x01		; initialize XH:XL to
	ldi S0, 0x10		; expanded key and call

	rcall addroundkey	; AddRoundKey

	first_addroundmask:
	ldi YH, 0x02		; points to the mask location
	ldi YL, 0x00		; so that to blind the key with the mask

	ldi XH, 0x03		; points to the offset array location
	ldi XL, 0x00
	clr H2

	rcall addroundmask 

	ldi ZH, hi8(__plain__)	; load plain text into ST11-ST44
	ldi ZL, lo8(__plain__)

	ld H1, Z+
	eor ST11, H1
	ld H1, Z+
	eor ST21, H1
	ld H1, Z+
	eor ST31, H1
	ld H1, Z+
	eor ST41, H1
	ld H1, Z+
	eor ST12, H1
	ld H1, Z+
	eor ST22, H1
	ld H1, Z+
	eor ST32, H1
	ld H1, Z+
	eor ST42, H1
	ld H1, Z+
	eor ST13, H1
	ld H1, Z+
	eor ST23, H1
	ld H1, Z+
	eor ST33, H1
	ld H1, Z+
	eor ST43, H1
	ld H1, Z+
	eor ST14, H1
	ld H1, Z+
	eor ST24, H1
	ld H1, Z+
	eor ST34, H1
	ld H1, Z+
	eor ST44, H1



encryp1:

	sbox_layer:

	; sbox(tab_adresse_sbox(tab_adresse_adress_sbox(i))+I2)
	; mask(tab_adresse_mask(tab_adresse_adresse_mask(i))+I2) 
	
	test1:
	ldi H1,1
	cp I, H1
	breq save_state
	test2:
	tst I2
	breq save_state
	round_fast:
	rjmp fast_sboxmc

	save_state:
	ldi XH, hi8(__plain__)
	ldi XL, lo8(__plain__)
	st X+, ST11
	st X+, ST21
	st X+, ST31
	st X+, ST41
	st X+, ST12
	st X+, ST22
	st X+, ST32
	st X+, ST42
	st X+, ST13
	st X+, ST23
	st X+, ST33
	st X+, ST43
	st X+, ST14
	st X+, ST24
	st X+, ST34
	st X+, ST44


	ldi XL,0x80
	tst I
	breq shuffled_sboxmc
	ldi XL, 0x30

	shuffled_sboxmc:
	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	ldi XH, hi8(__shuffle__)
	ldi YH,hi8(__offset__)
	ldi YL, lo8(__offset__)
	ld H2, X+
	add YL, H2
	ldi ZH, hi8(aes_sbox0)
	ld offset, Y
	add offset, I2
	andi offset, 0x0F
	add ZH, offset
	clr ZL
	ldi YH, hi8(__plain__)
	ldi YL, lo8(__plain__)
	add YL, H2
	ld ZL, Y
	clr H1
	lpm H1, Z
	ldi YH, hi8(__reg_out__)
	ldi YL, lo8(__reg_out__)
	add YL, H2
	ld ZL, Y
	clr ZH
	st Z, H1

	rjmp end_sboxmc



	fast_sboxmc:

	; sbox(tab_adresse_sbox(tab_adresse_adress_sbox(i))+I2)
	; mask(tab_adresse_mask(tab_adresse_adresse_mask(i))+I2) 
	ldi YH,hi8(__offset__) ; point to the offset array location
	ldi YL,0x00

	ld offset, Y ; offset = @Y[round + i]

	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL ; prevent from 1st order HD leackage (x + k + m_i) <- sbox(x + k) + m_i+1
	mov ZL, ST11  
	clr ST11
	lpm ST11, Z


	ldi YL, 4
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST12
	clr ST12
	lpm ST12, Z


	ldi YL, 8
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST13
	clr ST13
	lpm ST13, Z


	ldi YL, 12
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST14
	clr ST14
	lpm ST14, Z

	mov H1, ST21

	
	ldi YL, 5
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST22
	clr ST21
	lpm ST21, Z

	
	ldi YL, 9
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST23
	clr ST22
	lpm ST22, Z


	ldi YL, 13
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST24
	clr ST23
	lpm ST23, Z

	
	ldi YL, 1
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, H1
	clr ST24
	lpm ST24, Z

	mov H1, ST31

	ldi YL, 10
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST33
	clr ST31
	lpm ST31, Z

	ldi YL, 2
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, H1
	clr ST33
	lpm ST33, Z

	mov H1, ST32
	
	ldi YL, 14
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST34
	clr ST32
	lpm ST32, Z

	ldi YL, 6
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, H1
	clr ST34
	lpm ST34, Z

	mov H1, ST44
	
	ldi YL, 11
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST43
	clr ST44
	lpm ST44, Z

	ldi YL, 7
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST42
	clr ST43
	lpm ST43, Z


	ldi YL, 3
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, ST41
	clr ST42
	lpm ST42, Z


	ldi YL, 15
	ld offset, Y
	ldi ZH, hi8(aes_sbox0)
	add offset,I2 ;
	andi offset,0x0F
	add ZH, offset
	clr ZL
	mov ZL, H1
	clr ST41
	lpm ST41, Z

	end_sboxmc:
	inc I2
	dec I
	
	tst I
	breq ark
	rcall mixcolumns	; do MixColumns 

	ark:
	rcall addroundkey	; do AddRoundKey

	rcall prepare_pok_table
	rcall addroundpok	; do mask compensation

	tst I
	brne loop 
	ret

	loop:
	rjmp encryp1


;;; ***************************************************************************
;;; 
;;; ADDROUNDKEY
;;; This routine adds a round key to the __plain__ matrix (AddRoundKey). 
;;;
;;; Note: This routine is part of the encryption and decryption routines. You
;;; normally won't be interested in calling this routine directly.
;;;
;;; Parameters:
;;;         ST11-ST44:  128 bit __plain__ matrix
;;;         YH:YL:      pointer to ram location
;;; Touched registers:
;;;     ST11-ST41,H1,YH,YL

addroundkey:

	mov XL, S0
	mov XH, S1

	ld H1, X+
	eor ST11, H1
	ld H1, X+
	eor ST21, H1
	ld H1, X+
	eor ST31, H1
	ld H1, X+
	eor ST41, H1
	ld H1, X+
	eor ST12, H1
	ld H1, X+
	eor ST22, H1
	ld H1, X+
	eor ST32, H1
	ld H1, X+
	eor ST42, H1
	ld H1, X+
	eor ST13, H1
	ld H1, X+
	eor ST23, H1
	ld H1, X+
	eor ST33, H1
	ld H1, X+
	eor ST43, H1
	ld H1, X+
	eor ST14, H1
	ld H1, X+
	eor ST24, H1
	ld H1, X+
	eor ST34, H1
	ld H1, X+
	eor ST44, H1

	mov S0, XL
	mov S1, XH

	ret

addroundmask:

	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST11, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST21, H1
	
	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST31, H1
	
	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST41, H1


	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST12, H1
	
	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST22, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST32, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST42, H1


	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST13, H1
	
	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST23, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST33, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST43, H1
		
	
	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST14, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST24, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST34, H1

	mov YL, H2
	ld H1, X+
	add YL, H1
	andi YL, 0x0F
	ld H1, Y
	eor ST44, H1

	ret

prepare_pok_table:

	push_range 0, 16

	ldi XH, 0x02		; now, we point to the mask location
	ldi XL, 0x00		; so that to blind the key with the mask
	add XL, I2
	mov H2, XL

	ldi YH,hi8(__offset__) ; point to the offset array location
	ldi YL, 0x00

	; ShiftRows ( Mask[offset[i] + 1 + I2] , I2= round number )
	; mask(adresse_mask(adresse_adresse_mask(i))+I2) 
	ld H1, Y
	add XL, H1
	andi XL, 0x0F 
	clr ST11 ; this can be removed : HD(SR(MC(SBOX(k \oplus M , M) here.
	ld ST11, X

	mov XL, H2
	ldd H1, Y+4
	add XL, H1
	andi XL, 0x0F
	clr ST12
	ld ST12, X
	
	mov XL, H2
	ldd H1, Y+8
	add XL, H1
	andi XL, 0x0F
	clr ST13
	ld ST13, X 

	mov XL, H2
	ldd H1, Y+12
	add XL, H1
	andi XL, 0x0F
	clr ST14
	ld ST14, X

	mov XL, H2
	ldd H1, Y+5
	add XL, H1
	andi XL, 0x0F
	clr ST21
	ld ST21, X

	mov XL, H2
	ldd H1, Y+9
	add XL, H1
	andi XL, 0x0F
	clr ST22
	ld ST22, X

	mov XL, H2
	ldd H1, Y+13
	add XL, H1
	andi XL, 0x0F
	clr ST23
	ld ST23, X

	mov XL, H2
	ldd H1, Y+1
	add XL, H1
	andi XL, 0x0F
	clr ST24
	ld ST24, X

	mov XL, H2
	ldd H1, Y+10
	add XL, H1
	andi XL, 0x0F
	clr ST31
	ld ST31, X

	mov XL, H2
	ldd H1, Y+2
	add XL, H1
	andi XL, 0x0F
	clr ST33
	ld ST33, X

	mov XL, H2
	ldd H1, Y+14
	add XL, H1
	andi XL, 0x0F
	clr ST32
	ld ST32, X
	
	mov XL, H2
	ldd H1, Y+6
	add XL, H1
	andi XL, 0x0F
	clr ST34
	ld ST34, X

	mov XL, H2
	ldd H1, Y+11
	add XL, H1
	andi XL, 0x0F
	clr ST44
	ld ST44, X

	mov XL, H2
	ldd H1, Y+7
	add XL, H1
	andi XL, 0x0F
	ld ST43, X

	mov XL, H2
	ldd H1, Y+3
	add XL, H1
	andi XL, 0x0F
	ld ST42, X

	mov XL, H2
	ldd H1, Y+15
	add XL, H1
	andi XL, 0x0F
	clr ST41
	ld ST41, X

	tst I ; remove MixColums(mask) at the last round.
	breq save_pok


	rcall mixcolumns ; MixColums(ShiftRows ( Mask[offset[i] + 1 + I2] , I2= round number ) )
	pok_xor_mask:
	ldi YH, 0x02 ; mask location	
	ldi YL, 0x00		

	ldi XH, 0x03 ; point to the offset array location
	ldi XL, 0x00

	add YL, I2
	andi YL, 0x0F
	mov H2,YL
	rcall addroundmask

	; Save pok in ram (this operation can be optimised)
	save_pok:			; Stores new pok in ram 

	ldi YH, 0x02		; set the ram pointer next the key block 
	ldi YL, 0x90		

	st Y+, ST11
	st Y+, ST21
	st Y+, ST31
	st Y+, ST41
	st Y+, ST12
	st Y+, ST22
	st Y+, ST32
	st Y+, ST42
	st Y+, ST13
	st Y+, ST23
	st Y+, ST33
	st Y+, ST43
	st Y+, ST14
	st Y+, ST24
	st Y+, ST34
	st Y+, ST44

	; restore __state__ register
	pop_range 0, 16 ; 

	ret

addroundpok:

	ldi ZH, 0x02		; point to the pok table location
	ldi ZL, 0x90		; To check

	
	1:ld H1, Z+
	eor ST11, H1
	ld H1, Z+
	eor ST21, H1
	ld H1, Z+
	eor ST31, H1
	ld H1, Z+
	eor ST41, H1
	ld H1, Z+
	eor ST12, H1
	ld H1, Z+
	eor ST22, H1
	ld H1, Z+
	eor ST32, H1
	ld H1, Z+
	eor ST42, H1
	ld H1, Z+
	eor ST13, H1
	ld H1, Z+
	eor ST23, H1
	ld H1, Z+
	eor ST33, H1
	ld H1, Z+
	eor ST43, H1
	ld H1, Z+
	eor ST14, H1
	ld H1, Z+
	eor ST24, H1
	ld H1, Z+
	eor ST34, H1
	ld H1, Z+
	eor ST44, H1
	ret

;;; ***************************************************************************
;;; 
;;; MIXCOLUMNS
;;; This routine applies the MixColumns diffusion operator to the whole 
;;; state matrix. The code is used for both encryption and decryption.
;;;
;;; Note: This routine is part of the encryption and decryption routines. You
;;; normally wont be interested in calling this routine directly.
;;; 
;;; Parameters:
;;;         ST11-ST44:  128 bit __state__ matrix
;;; Touched registers:
;;;     ST11-ST41,H1,H2,H3,ZH,ZL

mixcolumns:
	ldi ZH, hi8(xtime)
	mov H1, ST11
	eor H1, ST21
	eor H1, ST31
	eor H1, ST41
	mov H2, ST11
	mov H3, ST11
	eor H3, ST21
	mov ZL, H3
	lpm H3, Z
	eor ST11, H3
	eor ST11, H1
	mov H3, ST21
	eor H3, ST31
	mov ZL, H3
	lpm H3, Z
	eor ST21, H3
	eor ST21, H1
	mov H3, ST31
	eor H3, ST41
	mov ZL, H3
	lpm H3, Z
	eor ST31, H3
	eor ST31, H1
	mov H3, ST41
	eor H3, H2
	mov ZL, H3
	lpm H3, Z
	eor ST41, H3
	eor ST41, H1
	
	mov H1, ST12
	eor H1, ST22
	eor H1, ST32
	eor H1, ST42
	mov H2, ST12
	mov H3, ST12
	eor H3, ST22
	mov ZL, H3
	lpm H3, Z
	eor ST12, H3
	eor ST12, H1
	mov H3, ST22
	eor H3, ST32
	mov ZL, H3
	lpm H3, Z
	eor ST22, H3
	eor ST22, H1
	mov H3, ST32
	eor H3, ST42
	mov ZL, H3
	lpm H3, Z
	eor ST32, H3
	eor ST32, H1
	mov H3, ST42
	eor H3, H2
	mov ZL, H3
	lpm H3, Z
	eor ST42, H3
	eor ST42, H1
	
	mov H1, ST13
	eor H1, ST23
	eor H1, ST33
	eor H1, ST43
	mov H2, ST13
	mov H3, ST13
	eor H3, ST23
	mov ZL, H3
	lpm H3, Z
	eor ST13, H3
	eor ST13, H1
	mov H3, ST23
	eor H3, ST33
	mov ZL, H3
	lpm H3, Z
	eor ST23, H3
	eor ST23, H1
	mov H3, ST33
	eor H3, ST43
	mov ZL, H3
	lpm H3, Z
	eor ST33, H3
	eor ST33, H1
	mov H3, ST43
	eor H3, H2
	mov ZL, H3
	lpm H3, Z
	eor ST43, H3
	eor ST43, H1
	
	mov H1, ST14
	eor H1, ST24
	eor H1, ST34
	eor H1, ST44
	mov H2, ST14
	mov H3, ST14
	eor H3, ST24
	mov ZL, H3
	lpm H3, Z
	eor ST14, H3
	eor ST14, H1
	mov H3, ST24
	eor H3, ST34
	mov ZL, H3
	lpm H3, Z
	eor ST24, H3
	eor ST24, H1
	mov H3, ST34
	eor H3, ST44
	mov ZL, H3
	lpm H3, Z
	eor ST34, H3
	eor ST34, H1
	mov H3, ST44
	eor H3, H2
	mov ZL, H3
	lpm H3, Z
	eor ST44, H3
	eor ST44, H1
	ret

;;; ***************************************************************************
;;; 
;;; S-BOX and "xtime" tables
;;;
;;; **************************************************************************
/*RSM*/
.balign 0x800


/*NON RSM*/

.global sbox	
sbox:
.byte 0x63,0x7c,0x77,0x7b,0xf2,0x6b,0x6f,0xc5,0x30,0x01,0x67,0x2b,0xfe,0xd7,0xab,0x76 
.byte 0xca,0x82,0xc9,0x7d,0xfa,0x59,0x47,0xf0,0xad,0xd4,0xa2,0xaf,0x9c,0xa4,0x72,0xc0 
.byte 0xb7,0xfd,0x93,0x26,0x36,0x3f,0xf7,0xcc,0x34,0xa5,0xe5,0xf1,0x71,0xd8,0x31,0x15 
.byte 0x04,0xc7,0x23,0xc3,0x18,0x96,0x05,0x9a,0x07,0x12,0x80,0xe2,0xeb,0x27,0xb2,0x75 
.byte 0x09,0x83,0x2c,0x1a,0x1b,0x6e,0x5a,0xa0,0x52,0x3b,0xd6,0xb3,0x29,0xe3,0x2f,0x84 
.byte 0x53,0xd1,0x00,0xed,0x20,0xfc,0xb1,0x5b,0x6a,0xcb,0xbe,0x39,0x4a,0x4c,0x58,0xcf 
.byte 0xd0,0xef,0xaa,0xfb,0x43,0x4d,0x33,0x85,0x45,0xf9,0x02,0x7f,0x50,0x3c,0x9f,0xa8 
.byte 0x51,0xa3,0x40,0x8f,0x92,0x9d,0x38,0xf5,0xbc,0xb6,0xda,0x21,0x10,0xff,0xf3,0xd2 
.byte 0xcd,0x0c,0x13,0xec,0x5f,0x97,0x44,0x17,0xc4,0xa7,0x7e,0x3d,0x64,0x5d,0x19,0x73 
.byte 0x60,0x81,0x4f,0xdc,0x22,0x2a,0x90,0x88,0x46,0xee,0xb8,0x14,0xde,0x5e,0x0b,0xdb 
.byte 0xe0,0x32,0x3a,0x0a,0x49,0x06,0x24,0x5c,0xc2,0xd3,0xac,0x62,0x91,0x95,0xe4,0x79 
.byte 0xe7,0xc8,0x37,0x6d,0x8d,0xd5,0x4e,0xa9,0x6c,0x56,0xf4,0xea,0x65,0x7a,0xae,0x08 
.byte 0xba,0x78,0x25,0x2e,0x1c,0xa6,0xb4,0xc6,0xe8,0xdd,0x74,0x1f,0x4b,0xbd,0x8b,0x8a 
.byte 0x70,0x3e,0xb5,0x66,0x48,0x03,0xf6,0x0e,0x61,0x35,0x57,0xb9,0x86,0xc1,0x1d,0x9e 
.byte 0xe1,0xf8,0x98,0x11,0x69,0xd9,0x8e,0x94,0x9b,0x1e,0x87,0xe9,0xce,0x55,0x28,0xdf 
.byte 0x8c,0xa1,0x89,0x0d,0xbf,0xe6,0x42,0x68,0x41,0x99,0x2d,0x0f,0xb0,0x54,0xbb,0x16 



xtime:
.byte 0x00,0x02,0x04,0x06,0x08,0x0a,0x0c,0x0e,0x10,0x12,0x14,0x16,0x18,0x1a,0x1c,0x1e
.byte 0x20,0x22,0x24,0x26,0x28,0x2a,0x2c,0x2e,0x30,0x32,0x34,0x36,0x38,0x3a,0x3c,0x3e
.byte 0x40,0x42,0x44,0x46,0x48,0x4a,0x4c,0x4e,0x50,0x52,0x54,0x56,0x58,0x5a,0x5c,0x5e
.byte 0x60,0x62,0x64,0x66,0x68,0x6a,0x6c,0x6e,0x70,0x72,0x74,0x76,0x78,0x7a,0x7c,0x7e
.byte 0x80,0x82,0x84,0x86,0x88,0x8a,0x8c,0x8e,0x90,0x92,0x94,0x96,0x98,0x9a,0x9c,0x9e
.byte 0xa0,0xa2,0xa4,0xa6,0xa8,0xaa,0xac,0xae,0xb0,0xb2,0xb4,0xb6,0xb8,0xba,0xbc,0xbe
.byte 0xc0,0xc2,0xc4,0xc6,0xc8,0xca,0xcc,0xce,0xd0,0xd2,0xd4,0xd6,0xd8,0xda,0xdc,0xde
.byte 0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xee,0xf0,0xf2,0xf4,0xf6,0xf8,0xfa,0xfc,0xfe
.byte 0x1b,0x19,0x1f,0x1d,0x13,0x11,0x17,0x15,0x0b,0x09,0x0f,0x0d,0x03,0x01,0x07,0x05
.byte 0x3b,0x39,0x3f,0x3d,0x33,0x31,0x37,0x35,0x2b,0x29,0x2f,0x2d,0x23,0x21,0x27,0x25
.byte 0x5b,0x59,0x5f,0x5d,0x53,0x51,0x57,0x55,0x4b,0x49,0x4f,0x4d,0x43,0x41,0x47,0x45
.byte 0x7b,0x79,0x7f,0x7d,0x73,0x71,0x77,0x75,0x6b,0x69,0x6f,0x6d,0x63,0x61,0x67,0x65
.byte 0x9b,0x99,0x9f,0x9d,0x93,0x91,0x97,0x95,0x8b,0x89,0x8f,0x8d,0x83,0x81,0x87,0x85
.byte 0xbb,0xb9,0xbf,0xbd,0xb3,0xb1,0xb7,0xb5,0xab,0xa9,0xaf,0xad,0xa3,0xa1,0xa7,0xa5
.byte 0xdb,0xd9,0xdf,0xdd,0xd3,0xd1,0xd7,0xd5,0xcb,0xc9,0xcf,0xcd,0xc3,0xc1,0xc7,0xc5
.byte 0xfb,0xf9,0xff,0xfd,0xf3,0xf1,0xf7,0xf5,0xeb,0xe9,0xef,0xed,0xe3,0xe1,0xe7,0xe5

.global aes_sbox0
aes_sbox0:

.byte 0x77, 0x7b, 0x70, 0x6f, 0xc9, 0x63, 0x67, 0xfe, 0x27, 0x6b, 0x0d, 0x3c, 0x7a, 0xa7, 0xdb, 0xf2
.byte 0x71, 0xc5, 0x8e, 0xc6, 0xfc, 0x4b, 0x55, 0xf6, 0xa3, 0xae, 0xd8, 0xa1, 0xcc, 0x7e, 0xa8, 0x90
.byte 0x2a, 0x9f, 0xf1, 0xbb, 0xc0, 0xfb, 0x33, 0x3a, 0xfd, 0xe9, 0xa9, 0x38, 0x19, 0x3d, 0xd4, 0x7d
.byte 0xcf, 0x2f, 0xcb, 0x08, 0x96, 0x09, 0x9a, 0x14, 0xee, 0x8c, 0x1e, 0x0b, 0x79, 0xbe, 0x2b, 0xe7
.byte 0x16, 0x20, 0x8f, 0x05, 0xac, 0x56, 0x62, 0x17, 0xbf, 0xda, 0x37, 0x5e, 0x88, 0x23, 0xef, 0x25
.byte 0xe1, 0x0c, 0xdd, 0x5f, 0x57, 0xbd, 0xf0, 0x2c, 0x35, 0xb2, 0xc7, 0x66, 0xc3, 0x54, 0x40, 0x46
.byte 0xf7, 0xa6, 0xe3, 0xdc, 0x89, 0x3f, 0x41, 0x4f, 0x73, 0x0e, 0xf5, 0x49, 0xa4, 0x93, 0x30, 0x5c
.byte 0x83, 0x4c, 0xaf, 0x5d, 0xf9, 0x34, 0x91, 0x9e, 0x2d, 0xd6, 0xba, 0xb0, 0xde, 0xff, 0xf3, 0x1c
.byte 0xe0, 0x1f, 0x00, 0xc1, 0x1b, 0x48, 0x9b, 0x53, 0x31, 0x72, 0xab, 0xc8, 0x7f, 0x15, 0x51, 0x68
.byte 0xd0, 0x43, 0x8d, 0x6c, 0x84, 0x9c, 0x26, 0x2e, 0x18, 0xb4, 0xe2, 0x4a, 0xd7, 0x07, 0x52, 0xd2
.byte 0x06, 0x36, 0x3e, 0xec, 0x50, 0x28, 0x0a, 0x45, 0x6e, 0xa0, 0xdf, 0xce, 0x75, 0xe8, 0x99, 0x9d
.byte 0x61, 0x3b, 0xc4, 0xeb, 0xa5, 0x42, 0xd9, 0x81, 0xe6, 0xf8, 0x5a, 0x60, 0x04, 0xa2, 0x76, 0x69
.byte 0x22, 0x29, 0x74, 0xb6, 0xca, 0xb8, 0xaa, 0x10, 0x13, 0x78, 0xd1, 0xe4, 0x86, 0x87, 0xb1, 0x47
.byte 0x6a, 0xb9, 0x32, 0x7c, 0x02, 0xfa, 0x0f, 0x44, 0xb5, 0x5b, 0x39, 0x6d, 0x92, 0x11, 0xcd, 0x8a
.byte 0x1d, 0x94, 0xf4, 0xed, 0x98, 0x82, 0xd5, 0x65, 0xe5, 0x8b, 0x12, 0x97, 0xd3, 0x24, 0x59, 0xc2
.byte 0x01, 0x85, 0xad, 0x80, 0x64, 0x4e, 0xea, 0xb3, 0x03, 0x21, 0x95, 0x4d, 0x1a, 0xb7, 0x58, 0xbc

.byte 0xcb, 0xe2, 0x9e, 0x43, 0x05, 0x34, 0x52, 0x1e, 0xc7, 0x5e, 0x5a, 0xf0, 0x56, 0x49, 0x42, 0x4e
.byte 0xa9, 0x91, 0x47, 0xf5, 0x98, 0xe1, 0x97, 0x9a, 0xcf, 0x6c, 0x72, 0xc5, 0xff, 0xb7, 0xfc, 0x48
.byte 0x44, 0xed, 0x04, 0x20, 0x01, 0x90, 0xd0, 0xc4, 0x03, 0x0a, 0xc2, 0xf9, 0x82, 0xc8, 0xa6, 0x13
.byte 0xde, 0x12, 0x87, 0x40, 0x32, 0x27, 0xb5, 0xd7, 0x2d, 0xa3, 0x30, 0xaf, 0x31, 0xf2, 0x16, 0xf6
.byte 0x1c, 0xd6, 0x1a, 0xb1, 0x67, 0x0e, 0xe3, 0x86, 0x2e, 0x5b, 0x6f, 0x95, 0x3c, 0xb6, 0x19, 0x2f
.byte 0x7f, 0x79, 0x6d, 0xfa, 0x5f, 0xfe, 0x8b, 0x0c, 0x15, 0xc9, 0x84, 0x6e, 0x66, 0xe4, 0x35, 0xd8
.byte 0x65, 0x09, 0xaa, 0x9d, 0x70, 0xcc, 0x37, 0x4a, 0x76, 0x78, 0x06, 0xb0, 0xe5, 0xda, 0x9f, 0xce
.byte 0x25, 0xca, 0xc6, 0xe7, 0x89, 0x83, 0xef, 0x14, 0xa7, 0xa8, 0x0d, 0xc0, 0x64, 0x96, 0x75, 0xba
.byte 0x51, 0x68, 0x2c, 0x46, 0xf1, 0x92, 0x4b, 0x08, 0x6a, 0xa2, 0x71, 0x22, 0xf8, 0x39, 0x26, 0xd9
.byte 0xeb, 0x6b, 0x3e, 0xee, 0x73, 0xdb, 0x8d, 0x21, 0x17, 0x1f, 0xa5, 0xbd, 0x55, 0xb4, 0x7a, 0xe9
.byte 0xa4, 0xa0, 0xd1, 0x4c, 0xf7, 0xe6, 0x99, 0x57, 0x7c, 0x33, 0x11, 0x69, 0xd5, 0x07, 0x0f, 0x3f
.byte 0x50, 0x4f, 0x9b, 0x3d, 0x59, 0x63, 0xc1, 0xdf, 0xb8, 0xe0, 0x7b, 0x9c, 0xd2, 0xfd, 0x02, 0x58
.byte 0x7e, 0x88, 0xbe, 0xbf, 0xdd, 0xe8, 0x41, 0x2a, 0x29, 0x93, 0x81, 0xf3, 0x8f, 0x4d, 0x10, 0x1b
.byte 0xb3, 0xf4, 0x28, 0xab, 0x54, 0x00, 0x62, 0x8c, 0x7d, 0x36, 0xc3, 0x3b, 0x45, 0x0b, 0x80, 0x53
.byte 0xfb, 0x60, 0x1d, 0xea, 0xae, 0x2b, 0xb2, 0xdc, 0x5c, 0xec, 0xbb, 0xa1, 0xd4, 0xcd, 0xad, 0x24
.byte 0x85, 0x61, 0x8e, 0x23, 0x74, 0xac, 0x18, 0x3a, 0x8a, 0xd3, 0x77, 0x5d, 0xb9, 0x94, 0xbc, 0x38

.byte 0xac, 0x22, 0xa0, 0x3f, 0xfd, 0x3e, 0xf9, 0x19, 0x1d, 0xd1, 0x4f, 0x88, 0x28, 0x3d, 0xd8, 0xba
.byte 0x05, 0x0c, 0xf6, 0xcd, 0xc7, 0x8d, 0x1c, 0xa9, 0xe2, 0x4b, 0x2f, 0x0b, 0x9f, 0x0e, 0xcb, 0xdf
.byte 0x63, 0xc0, 0xca, 0x7d, 0xb8, 0xf0, 0x47, 0xf3, 0x9e, 0xa6, 0xfa, 0x48, 0xee, 0x97, 0x95, 0x98
.byte 0x51, 0xc8, 0xff, 0x55, 0x46, 0x59, 0x41, 0x4d, 0xed, 0xc4, 0x4c, 0x91, 0x3b, 0x0a, 0x11, 0x5d
.byte 0xa7, 0xa8, 0xcf, 0x02, 0x99, 0x6b, 0xb5, 0x7a, 0xc5, 0x2a, 0xe8, 0xc9, 0x8c, 0x86, 0x1b, 0xe0
.byte 0x77, 0x79, 0xbf, 0x09, 0xd5, 0xea, 0xc1, 0x90, 0x06, 0x6a, 0x92, 0xa5, 0xc3, 0x7f, 0x45, 0x38
.byte 0xc6, 0x1a, 0x61, 0x8b, 0xeb, 0x69, 0xd7, 0x3a, 0x76, 0x70, 0xf5, 0x62, 0xf1, 0x50, 0x03, 0x84
.byte 0x54, 0x21, 0x9a, 0x60, 0xb9, 0x33, 0x20, 0x16, 0xd9, 0x13, 0xbe, 0x15, 0x01, 0x68, 0x89, 0xec
.byte 0xef, 0xb7, 0x93, 0x74, 0xf2, 0xdd, 0x57, 0x0d, 0x40, 0x5f, 0x32, 0x94, 0x6c, 0x56, 0xd0, 0xce
.byte 0x3c, 0x73, 0x66, 0x1e, 0x08, 0xda, 0x30, 0x00, 0xaf, 0xab, 0x43, 0xde, 0xe9, 0xf8, 0x58, 0x96
.byte 0x10, 0x18, 0xb2, 0xaa, 0xbb, 0x5a, 0xe6, 0x75, 0x64, 0xe4, 0xe1, 0x31, 0xd4, 0x7c, 0x2e, 0x82
.byte 0xad, 0x65, 0x2d, 0x7e, 0x36, 0xf7, 0xd6, 0x29, 0x67, 0x5e, 0x49, 0x23, 0x9d, 0xfe, 0x07, 0x44
.byte 0xdc, 0x85, 0x52, 0x78, 0x9b, 0xb6, 0x37, 0xb3, 0x6e, 0x8a, 0x2c, 0x81, 0xa3, 0x7b, 0x35, 0x17
.byte 0xe3, 0x53, 0xae, 0xb4, 0xc2, 0xdb, 0x2b, 0xa2, 0x6f, 0xf4, 0xe5, 0x12, 0x24, 0xa1, 0xd3, 0xbd
.byte 0x39, 0x72, 0x34, 0xcc, 0x04, 0x4a, 0x5c, 0x8f, 0xfb, 0xbc, 0xa4, 0x27, 0x0f, 0x5b, 0x83, 0x6d
.byte 0x9c, 0x26, 0xfc, 0x8e, 0x42, 0x80, 0x14, 0x1f, 0x87, 0x71, 0xb0, 0xb1, 0xe7, 0xd2, 0x25, 0x4e

.byte 0xd0, 0xb2, 0x57, 0x42, 0xe2, 0x25, 0xbb, 0x77, 0x73, 0x93, 0x54, 0x97, 0x55, 0xca, 0x48, 0xc6
.byte 0xb5, 0xa1, 0x64, 0xf5, 0x61, 0x45, 0x21, 0x88, 0xc3, 0x76, 0xe7, 0xad, 0xa7, 0x9c, 0x66, 0x6f
.byte 0xf2, 0xff, 0xfd, 0x84, 0x22, 0x90, 0xcc, 0xf4, 0x99, 0x2d, 0x9a, 0xd2, 0x17, 0xa0, 0xaa, 0x09
.byte 0x37, 0x7b, 0x60, 0x51, 0xfb, 0x26, 0xae, 0x87, 0x27, 0x2b, 0x33, 0x2c, 0x3f, 0x95, 0xa2, 0x3b
.byte 0x8a, 0x71, 0xec, 0xe6, 0xa3, 0x82, 0x40, 0xaf, 0x10, 0xdf, 0x01, 0xf3, 0x68, 0xa5, 0xc2, 0xcd
.byte 0x52, 0x2f, 0x15, 0xa9, 0xcf, 0xf8, 0x00, 0x6c, 0xfa, 0xab, 0x80, 0xbf, 0x63, 0xd5, 0x13, 0x1d
.byte 0xee, 0x69, 0x3a, 0x9b, 0x08, 0x9f, 0x1a, 0x1c, 0x50, 0xbd, 0x03, 0x81, 0xe1, 0x0b, 0x70, 0xac
.byte 0x86, 0xe3, 0x02, 0x6b, 0x7f, 0xd4, 0x79, 0xb3, 0x7c, 0x4a, 0x59, 0xd3, 0x0a, 0xf0, 0x4b, 0x3e
.byte 0xa4, 0xba, 0x3c, 0x06, 0xfe, 0x58, 0x35, 0x2a, 0x67, 0x3d, 0xb7, 0x98, 0x1e, 0xf9, 0xdd, 0x85
.byte 0xfc, 0x32, 0x92, 0x83, 0xb4, 0x29, 0xc1, 0xc5, 0x6a, 0x5a, 0xb0, 0x62, 0x74, 0x0c, 0x19, 0x56
.byte 0xe8, 0x44, 0x16, 0xbe, 0x5b, 0x8b, 0x8e, 0x0e, 0x1f, 0x8c, 0x30, 0xd1, 0xc0, 0xd8, 0x72, 0x7a
.byte 0x2e, 0x6d, 0x94, 0xf7, 0x49, 0x23, 0x34, 0x0d, 0x43, 0xbc, 0x9d, 0x5c, 0x14, 0x47, 0x0f, 0xc7
.byte 0x7d, 0x5f, 0x11, 0xc9, 0xeb, 0x46, 0xe0, 0x04, 0xd9, 0x5d, 0xdc, 0xf1, 0x12, 0x38, 0xef, 0xb6
.byte 0xd7, 0xb9, 0xcb, 0x4e, 0x78, 0x8f, 0x9e, 0x05, 0xc8, 0x41, 0xb1, 0xa8, 0xde, 0xc4, 0x39, 0x89
.byte 0x07, 0xe9, 0x31, 0x65, 0x4d, 0xce, 0xd6, 0x91, 0xe5, 0x36, 0x20, 0x6e, 0xa6, 0x5e, 0x18, 0x53
.byte 0x24, 0x4f, 0xb8, 0x8d, 0xdb, 0xda, 0x1b, 0xed, 0x75, 0x7e, 0xea, 0x28, 0xe4, 0x96, 0x4c, 0xf6

.byte 0x0c, 0x8e, 0x5f, 0xb2, 0x7f, 0xa3, 0xee, 0x04, 0x35, 0x94, 0xe1, 0x66, 0x15, 0x13, 0x07, 0x90
.byte 0x56, 0xdc, 0x73, 0x45, 0x44, 0x31, 0x05, 0xff, 0x0d, 0x64, 0x89, 0xec, 0x76, 0xbc, 0x70, 0xdb
.byte 0x0e, 0xfc, 0x1f, 0xd0, 0xcd, 0xc2, 0x67, 0xaa, 0xe3, 0xe9, 0x85, 0x7e, 0x4f, 0xa0, 0xac, 0x8d
.byte 0x8f, 0xb0, 0xf5, 0xa4, 0x1c, 0x12, 0x6c, 0xda, 0x1a, 0xa6, 0x5d, 0x20, 0x0f, 0x63, 0xc0, 0xf7
.byte 0x95, 0xdd, 0x96, 0x22, 0xa5, 0x06, 0x18, 0xaf, 0xf2, 0x8b, 0xfd, 0xf0, 0xc3, 0xfb, 0x2d, 0x9f
.byte 0x3c, 0x23, 0x28, 0x24, 0xad, 0x34, 0x30, 0x9a, 0x6f, 0x5e, 0x38, 0x74, 0xa1, 0x88, 0xf4, 0x29
.byte 0x5b, 0x98, 0x7c, 0x9c, 0x47, 0xc9, 0x5a, 0xc5, 0x58, 0x4d, 0xdf, 0xbd, 0xb4, 0x78, 0xed, 0x2a
.byte 0xe8, 0xa2, 0xcc, 0x79, 0x69, 0x60, 0xa8, 0x93, 0x6b, 0xfa, 0xba, 0xae, 0x2e, 0x87, 0x6e, 0x4a
.byte 0x2f, 0x61, 0xea, 0x39, 0x17, 0x5c, 0xa9, 0x51, 0x3e, 0x6a, 0x08, 0xe6, 0xd9, 0x9e, 0x42, 0xc1
.byte 0xe5, 0x27, 0x7a, 0x71, 0x43, 0xf9, 0xeb, 0x99, 0xb7, 0x82, 0x2b, 0x40, 0x14, 0xe2, 0xd4, 0xd5
.byte 0xd3, 0xfe, 0xd6, 0x52, 0xe0, 0xb9, 0x1d, 0x37, 0x1e, 0xc6, 0x72, 0x50, 0xef, 0x0b, 0xe4, 0x49
.byte 0xbe, 0xa7, 0xc7, 0x4e, 0x36, 0x86, 0xd1, 0xcb, 0xc4, 0x41, 0xd8, 0xb6, 0x91, 0x0a, 0x77, 0x80
.byte 0x3f, 0xde, 0x10, 0x83, 0x7d, 0x75, 0xcf, 0xd7, 0x19, 0xb1, 0xe7, 0x4b, 0x81, 0x01, 0x54, 0x84
.byte 0x92, 0x53, 0x4c, 0xb3, 0x00, 0xc8, 0x1b, 0x48, 0x9b, 0xf8, 0x21, 0x62, 0x3b, 0x02, 0x46, 0x2c
.byte 0xb8, 0x97, 0x68, 0x32, 0xd2, 0x8a, 0x11, 0xf6, 0x33, 0x09, 0xab, 0xb5, 0x3a, 0x25, 0xf1, 0x57
.byte 0xbf, 0x6d, 0x65, 0x55, 0x16, 0x59, 0x7b, 0x03, 0x9d, 0x8c, 0xf3, 0x3d, 0xce, 0xca, 0xbb, 0x26

.byte 0xa9, 0x3e, 0x2a, 0x2c, 0x5f, 0xd8, 0xad, 0x0c, 0x3d, 0xd7, 0x9a, 0x46, 0x8b, 0x66, 0xb7, 0x35
.byte 0xe2, 0x49, 0x85, 0x4f, 0xd5, 0xb0, 0x5d, 0x34, 0xc6, 0x3c, 0x08, 0x7d, 0x7c, 0x4a, 0xe5, 0x6f
.byte 0xb4, 0x95, 0x99, 0x76, 0x47, 0xbc, 0xd0, 0xda, 0x93, 0x5e, 0xfb, 0xf4, 0xe9, 0x26, 0xc5, 0x37
.byte 0xce, 0xf9, 0x5a, 0x36, 0x19, 0x64, 0x9f, 0x23, 0xe3, 0x55, 0x2b, 0x25, 0x9d, 0xcc, 0x89, 0xb6
.byte 0xa6, 0x14, 0xc2, 0xfa, 0xc9, 0xc4, 0xb2, 0xcb, 0x96, 0x21, 0x3f, 0x9c, 0x1b, 0xaf, 0xe4, 0xac
.byte 0x10, 0xcd, 0xb1, 0x98, 0x4d, 0x01, 0x67, 0x56, 0xa3, 0x09, 0x0d, 0x94, 0x1d, 0x11, 0x1a, 0x05
.byte 0x13, 0xd4, 0x41, 0x8d, 0x84, 0xe6, 0x74, 0x61, 0xfc, 0x63, 0xf0, 0x7e, 0xa5, 0x45, 0xa1, 0x62
.byte 0x73, 0x57, 0xbe, 0x17, 0x97, 0x83, 0xc3, 0x52, 0xaa, 0x91, 0x59, 0x50, 0x40, 0xf5, 0x9b, 0xd1
.byte 0xf8, 0x7b, 0xa7, 0xe0, 0xdf, 0x31, 0x53, 0x07, 0x68, 0x90, 0x65, 0x2e, 0x00, 0xd3, 0x58, 0x16
.byte 0xec, 0xed, 0xdb, 0x2d, 0x79, 0x12, 0xbb, 0x8e, 0xa0, 0xd2, 0xc0, 0x7a, 0x48, 0x43, 0x1e, 0xdc
.byte 0x70, 0xdd, 0x32, 0xd6, 0x69, 0x4b, 0xff, 0x27, 0x0e, 0x24, 0x80, 0xd9, 0x6b, 0xef, 0xc7, 0xea
.byte 0xb9, 0x4e, 0x33, 0xa8, 0x8f, 0xe1, 0x78, 0xfd, 0xf2, 0xe8, 0xbf, 0x0f, 0x77, 0xfe, 0x9e, 0x87
.byte 0xbd, 0x6d, 0x38, 0xb8, 0x72, 0xde, 0x88, 0x20, 0xee, 0xf6, 0x4c, 0x44, 0xba, 0x29, 0xe7, 0x06
.byte 0x15, 0x7f, 0x3b, 0x02, 0x5b, 0x18, 0xc1, 0xa2, 0x71, 0x22, 0xf1, 0x39, 0x8a, 0x75, 0x6a, 0xab
.byte 0x6e, 0xc8, 0x1c, 0x03, 0x8c, 0x92, 0x30, 0x0a, 0xcf, 0x28, 0xb3, 0xeb, 0x0b, 0x51, 0xae, 0x81
.byte 0x1f, 0x82, 0xf3, 0xf7, 0x04, 0xca, 0xb5, 0xa4, 0x3a, 0x42, 0x60, 0x2f, 0x6c, 0x5c, 0x54, 0x86

.byte 0x5a, 0xec, 0x2a, 0x24, 0xc3, 0x92, 0xb9, 0x86, 0xf6, 0xc1, 0x39, 0x55, 0x6b, 0x16, 0x2c, 0x90
.byte 0x51, 0x9c, 0xfb, 0xf4, 0x29, 0xe6, 0x38, 0xca, 0x9a, 0xbb, 0x79, 0x96, 0xb3, 0x48, 0xd5, 0xdf
.byte 0x33, 0xc9, 0x72, 0x07, 0x45, 0x73, 0x60, 0xea, 0x46, 0xed, 0x40, 0x8a, 0xbf, 0xda, 0x3b, 0x52
.byte 0xd8, 0x32, 0x49, 0x95, 0x69, 0x84, 0x3a, 0xb8, 0x31, 0xa6, 0x23, 0x25, 0xd7, 0x50, 0x03, 0xa2
.byte 0x9e, 0xa5, 0x5f, 0x56, 0xfa, 0x4f, 0xde, 0x94, 0x58, 0x7c, 0x18, 0xb1, 0x8c, 0x98, 0x5d, 0xcc
.byte 0x6c, 0xf3, 0x71, 0xff, 0x4a, 0xaa, 0x6d, 0xae, 0xdb, 0x1c, 0x82, 0x4e, 0xe9, 0x8b, 0x6e, 0x7b
.byte 0x06, 0xac, 0x9b, 0x02, 0x1e, 0x12, 0x0a, 0x15, 0xc2, 0x1f, 0x97, 0xbe, 0x0e, 0x42, 0x59, 0x68
.byte 0x2e, 0x99, 0x93, 0x30, 0xa0, 0x14, 0xa3, 0xeb, 0x1b, 0xa9, 0xf5, 0xcd, 0xcb, 0xc6, 0xc4, 0xbd
.byte 0xe7, 0xfd, 0x00, 0xb0, 0xf1, 0x78, 0x88, 0x91, 0x41, 0xb6, 0xa7, 0x3c, 0xee, 0x80, 0xf2, 0x77
.byte 0x2b, 0x01, 0xd6, 0x8f, 0xe0, 0x64, 0xe5, 0xc8, 0xd2, 0x7f, 0xd9, 0x3d, 0x44, 0x66, 0x28, 0xf0
.byte 0xdd, 0xaf, 0x75, 0xcf, 0x4c, 0x47, 0xd3, 0x11, 0xe2, 0xe3, 0x22, 0xd4, 0x1d, 0x76, 0x81, 0xb4
.byte 0x9f, 0x67, 0x21, 0x6a, 0xdc, 0x0f, 0x19, 0x57, 0x74, 0xf7, 0xef, 0xa8, 0x3e, 0xd0, 0x08, 0x5c
.byte 0x4d, 0x35, 0x20, 0x6f, 0x53, 0x63, 0x89, 0x5b, 0x8d, 0x10, 0xf8, 0xfc, 0xc5, 0x0b, 0xab, 0xba
.byte 0x27, 0xc0, 0xe4, 0xbc, 0x5e, 0x04, 0x8e, 0xa1, 0xc7, 0x61, 0x0c, 0x13, 0x9d, 0x83, 0x05, 0x3f
.byte 0x2d, 0x7e, 0x36, 0xfe, 0x7a, 0x85, 0xa4, 0x65, 0x70, 0x1a, 0x0d, 0x34, 0x17, 0x54, 0xad, 0xce
.byte 0xf9, 0xe1, 0x4b, 0x43, 0x26, 0xb5, 0x09, 0xe8, 0x62, 0xb2, 0xb7, 0x37, 0xd1, 0x7d, 0x2f, 0x87

.byte 0x6f, 0xd3, 0xe9, 0x94, 0xaa, 0xc6, 0x3e, 0x09, 0x79, 0x46, 0x6d, 0x3c, 0xdb, 0xd5, 0x13, 0xa5
.byte 0x20, 0x2a, 0xb7, 0x4c, 0x69, 0x86, 0x44, 0x65, 0x35, 0xc7, 0x19, 0xd6, 0x0b, 0x04, 0x63, 0xae
.byte 0xad, 0xc4, 0x25, 0x40, 0x75, 0xbf, 0x12, 0xb9, 0x15, 0x9f, 0x8c, 0xba, 0xf8, 0x8d, 0x36, 0xcc
.byte 0x5d, 0xfc, 0xaf, 0x28, 0xda, 0xdc, 0x59, 0xce, 0x47, 0xc5, 0x7b, 0x96, 0x6a, 0xb6, 0xcd, 0x27
.byte 0x33, 0xa2, 0x67, 0x73, 0x4e, 0xe7, 0x83, 0xa7, 0x6b, 0x21, 0xb0, 0x05, 0xa9, 0xa0, 0x5a, 0x61
.byte 0x84, 0x91, 0x74, 0x16, 0xb1, 0x7d, 0xe3, 0x24, 0x51, 0x92, 0x55, 0xb5, 0x00, 0x8e, 0x0c, 0x93
.byte 0x97, 0xa6, 0xbd, 0xf1, 0x41, 0x68, 0xe0, 0x3d, 0xea, 0xf5, 0xed, 0xe1, 0xfd, 0x64, 0x53, 0xf9
.byte 0x42, 0x3b, 0x39, 0x34, 0x32, 0x0a, 0x56, 0xe4, 0x14, 0x5c, 0xeb, 0x5f, 0xcf, 0x6c, 0x66, 0xd1
.byte 0x88, 0x0d, 0x7f, 0x11, 0xc3, 0x58, 0x49, 0xbe, 0x6e, 0x77, 0x87, 0x0e, 0x4f, 0xff, 0x02, 0x18
.byte 0x0f, 0xd7, 0x99, 0xbb, 0xc2, 0x26, 0x80, 0x2d, 0x37, 0x1a, 0x9b, 0x1f, 0x70, 0x29, 0xfe, 0xd4
.byte 0x4b, 0x7e, 0x89, 0xe2, 0x2b, 0xdd, 0x1c, 0x1d, 0xee, 0x2c, 0xb8, 0xb3, 0x30, 0x8a, 0x50, 0x22
.byte 0xa3, 0xf7, 0x2f, 0xc1, 0x57, 0x10, 0x08, 0x8b, 0xa8, 0xe6, 0xf0, 0x23, 0x95, 0xde, 0x98, 0x60
.byte 0x45, 0x54, 0xf4, 0x3a, 0x03, 0x07, 0xef, 0x72, 0xa4, 0x76, 0x9c, 0xac, 0x90, 0xdf, 0xca, 0xb2
.byte 0xc0, 0xfa, 0x7c, 0x62, 0xec, 0xf3, 0x9e, 0x38, 0x5e, 0x71, 0xfb, 0xa1, 0x43, 0x1b, 0x3f, 0xd8
.byte 0x31, 0x52, 0xab, 0xe8, 0xcb, 0xf2, 0xe5, 0x8f, 0x9a, 0x5b, 0x7a, 0x85, 0x01, 0xc9, 0x81, 0xd2
.byte 0x78, 0xd0, 0x82, 0x2e, 0xc8, 0x48, 0x4d, 0x9d, 0x17, 0xf6, 0x4a, 0xd9, 0xbc, 0xb4, 0x1e, 0x06

.byte 0x09, 0x11, 0xbb, 0xb3, 0xd6, 0x45, 0xf9, 0x18, 0x92, 0x42, 0x47, 0xc7, 0x21, 0x8d, 0xdf, 0x77
.byte 0xdd, 0x8e, 0xc6, 0x0e, 0x8a, 0x75, 0x54, 0x95, 0x80, 0xea, 0xfd, 0xc4, 0xe7, 0xa4, 0x5d, 0x3e
.byte 0xd7, 0x30, 0x14, 0x4c, 0xae, 0xf4, 0x7e, 0x51, 0x37, 0x91, 0xfc, 0xe3, 0x6d, 0x73, 0xf5, 0xcf
.byte 0xbd, 0xc5, 0xd0, 0x9f, 0xa3, 0x93, 0x79, 0xab, 0x7d, 0xe0, 0x08, 0x0c, 0x35, 0xfb, 0x5b, 0x4a
.byte 0x6f, 0x97, 0xd1, 0x9a, 0x2c, 0xff, 0xe9, 0xa7, 0x84, 0x07, 0x1f, 0x58, 0xce, 0x20, 0xf8, 0xac
.byte 0x2d, 0x5f, 0x85, 0x3f, 0xbc, 0xb7, 0x23, 0xe1, 0x12, 0x13, 0xd2, 0x24, 0xed, 0x86, 0x71, 0x44
.byte 0xdb, 0xf1, 0x26, 0x7f, 0x10, 0x94, 0x15, 0x38, 0x22, 0x8f, 0x29, 0xcd, 0xb4, 0x96, 0xd8, 0x00
.byte 0x17, 0x0d, 0xf0, 0x40, 0x01, 0x88, 0x78, 0x61, 0xb1, 0x46, 0x57, 0xcc, 0x1e, 0x70, 0x02, 0x87
.byte 0xde, 0x69, 0x63, 0xc0, 0x50, 0xe4, 0x53, 0x1b, 0xeb, 0x59, 0x05, 0x3d, 0x3b, 0x36, 0x34, 0x4d
.byte 0xf6, 0x5c, 0x6b, 0xf2, 0xee, 0xe2, 0xfa, 0xe5, 0x32, 0xef, 0x67, 0x4e, 0xfe, 0xb2, 0xa9, 0x98
.byte 0x9c, 0x03, 0x81, 0x0f, 0xba, 0x5a, 0x9d, 0x5e, 0x2b, 0xec, 0x72, 0xbe, 0x19, 0x7b, 0x9e, 0x8b
.byte 0x6e, 0x55, 0xaf, 0xa6, 0x0a, 0xbf, 0x2e, 0x64, 0xa8, 0x8c, 0xe8, 0x41, 0x7c, 0x68, 0xad, 0x3c
.byte 0x28, 0xc2, 0xb9, 0x65, 0x99, 0x74, 0xca, 0x48, 0xc1, 0x56, 0xd3, 0xd5, 0x27, 0xa0, 0xf3, 0x52
.byte 0xc3, 0x39, 0x82, 0xf7, 0xb5, 0x83, 0x90, 0x1a, 0xb6, 0x1d, 0xb0, 0x7a, 0x4f, 0x2a, 0xcb, 0xa2
.byte 0xa1, 0x6c, 0x0b, 0x04, 0xd9, 0x16, 0xc8, 0x3a, 0x6a, 0x4b, 0x89, 0x66, 0x43, 0xb8, 0x25, 0x2f
.byte 0xaa, 0x1c, 0xda, 0xd4, 0x33, 0x62, 0x49, 0x76, 0x06, 0x31, 0xc9, 0xa5, 0x9b, 0xe6, 0xdc, 0x60

.byte 0x4e, 0xe6, 0xb4, 0x18, 0xfe, 0x7e, 0x7b, 0xab, 0x21, 0xc0, 0x7c, 0xef, 0x8a, 0x82, 0x28, 0x30
.byte 0x07, 0x64, 0x9d, 0xde, 0xfd, 0xc4, 0xd3, 0xb9, 0xac, 0x6d, 0x4c, 0xb3, 0x37, 0xff, 0xb7, 0xe4
.byte 0xf6, 0xcc, 0x4a, 0x54, 0xda, 0xc5, 0xa8, 0x0e, 0x68, 0x47, 0xcd, 0x97, 0x75, 0x2d, 0x09, 0xee
.byte 0x73, 0x62, 0xc2, 0x0c, 0x35, 0x31, 0xd9, 0x44, 0x92, 0x40, 0xaa, 0x9a, 0xa6, 0xe9, 0xfc, 0x84
.byte 0x95, 0xc1, 0x19, 0xf7, 0x61, 0x26, 0x3e, 0xbd, 0x9e, 0xd0, 0xc6, 0x15, 0xa3, 0xe8, 0xae, 0x56
.byte 0x7d, 0x48, 0xbf, 0xd4, 0x1d, 0xeb, 0x2a, 0x2b, 0xd8, 0x1a, 0x8e, 0x85, 0x06, 0xbc, 0x66, 0x14
.byte 0x39, 0xe1, 0xaf, 0x8d, 0xf4, 0x10, 0xb6, 0x1b, 0x01, 0x2c, 0xad, 0x29, 0x46, 0x1f, 0xc8, 0xe2
.byte 0xbe, 0x3b, 0x49, 0x27, 0xf5, 0x6e, 0x7f, 0x88, 0x58, 0x41, 0xb1, 0x38, 0x79, 0xc9, 0x34, 0x2e
.byte 0x74, 0x0d, 0x0f, 0x02, 0x04, 0x3c, 0x60, 0xd2, 0x22, 0x6a, 0xdd, 0x69, 0xf9, 0x5a, 0x50, 0xe7
.byte 0xa1, 0x90, 0x8b, 0xc7, 0x77, 0x5e, 0xd6, 0x0b, 0xdc, 0xc3, 0xdb, 0xd7, 0xcb, 0x52, 0x65, 0xcf
.byte 0xb2, 0xa7, 0x42, 0x20, 0x87, 0x4b, 0xd5, 0x12, 0x67, 0xa4, 0x63, 0x83, 0x36, 0xb8, 0x3a, 0xa5
.byte 0x05, 0x94, 0x51, 0x45, 0x78, 0xd1, 0xb5, 0x91, 0x5d, 0x17, 0x86, 0x33, 0x9f, 0x96, 0x6c, 0x57
.byte 0x6b, 0xca, 0x99, 0x1e, 0xec, 0xea, 0x6f, 0xf8, 0x71, 0xf3, 0x4d, 0xa0, 0x5c, 0x80, 0xfb, 0x11
.byte 0x9b, 0xf2, 0x13, 0x76, 0x43, 0x89, 0x24, 0x8f, 0x23, 0xa9, 0xba, 0x8c, 0xce, 0xbb, 0x00, 0xfa
.byte 0x16, 0x1c, 0x81, 0x7a, 0x5f, 0xb0, 0x72, 0x53, 0x03, 0xf1, 0x2f, 0xe0, 0x3d, 0x32, 0x55, 0x98
.byte 0x59, 0xe5, 0xdf, 0xa2, 0x9c, 0xf0, 0x08, 0x3f, 0x4f, 0x70, 0x5b, 0x0a, 0xed, 0xe3, 0x25, 0x93

.byte 0x4f, 0x9d, 0x95, 0xa5, 0xe6, 0xa9, 0x8b, 0xf3, 0x6d, 0x7c, 0x03, 0xcd, 0x3e, 0x3a, 0x4b, 0xd6
.byte 0x48, 0x67, 0x98, 0xc2, 0x22, 0x7a, 0xe1, 0x06, 0xc3, 0xf9, 0x5b, 0x45, 0xca, 0xd5, 0x01, 0xa7
.byte 0x62, 0xa3, 0xbc, 0x43, 0xf0, 0x38, 0xeb, 0xb8, 0x6b, 0x08, 0xd1, 0x92, 0xcb, 0xf2, 0xb6, 0xdc
.byte 0xcf, 0x2e, 0xe0, 0x73, 0x8d, 0x85, 0x3f, 0x27, 0xe9, 0x41, 0x17, 0xbb, 0x71, 0xf1, 0xa4, 0x74
.byte 0x4e, 0x57, 0x37, 0xbe, 0xc6, 0x76, 0x21, 0x3b, 0x34, 0xb1, 0x28, 0x46, 0x61, 0xfa, 0x87, 0x70
.byte 0x23, 0x0e, 0x26, 0xa2, 0x10, 0x49, 0xed, 0xc7, 0xee, 0x36, 0x82, 0xa0, 0x1f, 0xfb, 0x14, 0xb9
.byte 0x15, 0xd7, 0x8a, 0x81, 0xb3, 0x09, 0x1b, 0x69, 0x47, 0x72, 0xdb, 0xb0, 0xe4, 0x12, 0x24, 0x25
.byte 0xdf, 0x91, 0x1a, 0xc9, 0xe7, 0xac, 0x59, 0xa1, 0xce, 0x9a, 0xf8, 0x16, 0x29, 0x6e, 0xb2, 0x31
.byte 0x18, 0x52, 0x3c, 0x89, 0x99, 0x90, 0x58, 0x63, 0x9b, 0x0a, 0x4a, 0x5e, 0xde, 0x77, 0x9e, 0xba
.byte 0xab, 0x68, 0x8c, 0x6c, 0xb7, 0x39, 0xaa, 0x35, 0xa8, 0xbd, 0x2f, 0x4d, 0x44, 0x88, 0x1d, 0xda
.byte 0xcc, 0xd3, 0xd8, 0xd4, 0x5d, 0xc4, 0xc0, 0x6a, 0x9f, 0xae, 0xc8, 0x84, 0x51, 0x78, 0x04, 0xd9
.byte 0x65, 0x2d, 0x66, 0xd2, 0x55, 0xf6, 0xe8, 0x5f, 0x02, 0x7b, 0x0d, 0x00, 0x33, 0x0b, 0xdd, 0x6f
.byte 0x7f, 0x40, 0x05, 0x54, 0xec, 0xe2, 0x9c, 0x2a, 0xea, 0x56, 0xad, 0xd0, 0xff, 0x93, 0x30, 0x07
.byte 0xfe, 0x0c, 0xef, 0x20, 0x3d, 0x32, 0x97, 0x5a, 0x13, 0x19, 0x75, 0x8e, 0xbf, 0x50, 0x5c, 0x7d
.byte 0xa6, 0x2c, 0x83, 0xb5, 0xb4, 0xc1, 0xf5, 0x0f, 0xfd, 0x94, 0x79, 0x1c, 0x86, 0x4c, 0x80, 0x2b
.byte 0xfc, 0x7e, 0xaf, 0x42, 0x8f, 0x53, 0x1e, 0xf4, 0xc5, 0x64, 0x11, 0x96, 0xe5, 0xe3, 0xf7, 0x60

.byte 0xbc, 0x21, 0x50, 0x54, 0xa7, 0x69, 0x16, 0x07, 0x99, 0xe1, 0xc3, 0x8c, 0xcf, 0xff, 0xf7, 0x25
.byte 0xcd, 0x6b, 0xbf, 0xa0, 0x2f, 0x31, 0x93, 0xa9, 0x6c, 0x8b, 0x10, 0x48, 0xa8, 0xf2, 0x0d, 0x22
.byte 0xb6, 0xdc, 0x98, 0xa1, 0xf8, 0xbb, 0x62, 0x01, 0xd2, 0x81, 0x52, 0x9a, 0x29, 0xd6, 0xc9, 0x08
.byte 0x1e, 0xce, 0x9b, 0x1b, 0xd1, 0x7d, 0x2b, 0x83, 0x4d, 0x55, 0xef, 0xe7, 0x19, 0x8a, 0x44, 0xa5
.byte 0x1a, 0xed, 0x90, 0x0b, 0x2c, 0x42, 0xdb, 0x5e, 0x51, 0x4b, 0x1c, 0xac, 0xd4, 0x5d, 0x3d, 0x24
.byte 0xd3, 0x7e, 0x91, 0x75, 0xca, 0xe8, 0x5c, 0x84, 0xad, 0x87, 0x23, 0x7a, 0xc8, 0x4c, 0x64, 0x49
.byte 0x4f, 0x4e, 0x78, 0x8e, 0xda, 0xb1, 0x18, 0x2d, 0x03, 0x71, 0x63, 0xd9, 0xeb, 0xe0, 0xbd, 0x7f
.byte 0x5b, 0xd8, 0x04, 0x43, 0x7c, 0x92, 0xf0, 0xa4, 0xcb, 0x33, 0xc6, 0x8d, 0xa3, 0x70, 0xfb, 0xb5
.byte 0xd0, 0xf4, 0x1d, 0xb4, 0x34, 0x20, 0x60, 0xf1, 0x09, 0x32, 0xfa, 0xf3, 0xe3, 0x56, 0x38, 0x72
.byte 0xb0, 0x77, 0xe2, 0x2e, 0x27, 0x45, 0xd7, 0xc2, 0x5f, 0xc0, 0x53, 0xdd, 0x06, 0xe6, 0x02, 0xc1
.byte 0xb3, 0x6e, 0x12, 0x3b, 0xee, 0xa2, 0xc4, 0xf5, 0x00, 0xaa, 0xae, 0x37, 0xbe, 0xb2, 0xb9, 0xa6
.byte 0x05, 0xb7, 0x61, 0x59, 0x6a, 0x67, 0x11, 0x68, 0x35, 0x82, 0x9c, 0x3f, 0xb8, 0x0c, 0x47, 0x0f
.byte 0x6d, 0x5a, 0xf9, 0x95, 0xba, 0xc7, 0x3c, 0x80, 0x40, 0xf6, 0x88, 0x86, 0x3e, 0x6f, 0x2a, 0x15
.byte 0x17, 0x36, 0x3a, 0xd5, 0xe4, 0x1f, 0x73, 0x79, 0x30, 0xfd, 0x58, 0x57, 0x4a, 0x85, 0x66, 0x94
.byte 0x41, 0xea, 0x26, 0xec, 0x76, 0x13, 0xfe, 0x97, 0x65, 0x9f, 0xab, 0xde, 0xdf, 0xe9, 0x46, 0xcc
.byte 0x0a, 0x9d, 0x89, 0x8f, 0xfc, 0x7b, 0x0e, 0xaf, 0x9e, 0x74, 0x39, 0xe5, 0x28, 0xc5, 0x14, 0x96

.byte 0x6c, 0xd6, 0x0c, 0x7e, 0xb2, 0x70, 0xe4, 0xef, 0x77, 0x81, 0x40, 0x41, 0x17, 0x22, 0xd5, 0xbe
.byte 0xc9, 0x82, 0xc4, 0x3c, 0xf4, 0xba, 0xac, 0x7f, 0x0b, 0x4c, 0x54, 0xd7, 0xff, 0xab, 0x73, 0x9d
.byte 0x13, 0xa3, 0x5e, 0x44, 0x32, 0x2b, 0xdb, 0x52, 0x9f, 0x04, 0x15, 0xe2, 0xd4, 0x51, 0x23, 0x4d
.byte 0x2c, 0x75, 0xa2, 0x88, 0x6b, 0x46, 0xc7, 0x43, 0x9e, 0x7a, 0xdc, 0x71, 0x53, 0x8b, 0xc5, 0xe7
.byte 0x5d, 0x95, 0xdd, 0x8e, 0xc6, 0x07, 0x26, 0xd9, 0x97, 0xae, 0xb9, 0xd3, 0x6d, 0x0e, 0xf7, 0xb4
.byte 0xe0, 0xe8, 0x42, 0x5a, 0x4b, 0xaa, 0x16, 0x85, 0x94, 0x14, 0x11, 0xc1, 0x24, 0x8c, 0xde, 0x72
.byte 0xcc, 0x83, 0x96, 0xee, 0xf8, 0x2a, 0xc0, 0xf0, 0x5f, 0x5b, 0xb3, 0x2e, 0x19, 0x08, 0xa8, 0x66
.byte 0x1f, 0x47, 0x63, 0x84, 0x02, 0x2d, 0xa7, 0xfd, 0xb0, 0xaf, 0xc2, 0x64, 0x9c, 0xa6, 0x20, 0x3e
.byte 0xa4, 0xd1, 0x6a, 0x90, 0x49, 0xc3, 0xd0, 0xe6, 0x29, 0xe3, 0x4e, 0xe5, 0xf1, 0x98, 0x79, 0x1c
.byte 0x36, 0xea, 0x91, 0x7b, 0x1b, 0x99, 0x27, 0xca, 0x86, 0x80, 0x05, 0x92, 0x01, 0xa0, 0xf3, 0x74
.byte 0x87, 0x89, 0x4f, 0xf9, 0x25, 0x1a, 0x31, 0x60, 0xf6, 0x9a, 0x62, 0x55, 0x33, 0x8f, 0xb5, 0xc8
.byte 0x57, 0x58, 0x3f, 0xf2, 0x69, 0x9b, 0x45, 0x8a, 0x35, 0xda, 0x18, 0x39, 0x7c, 0x76, 0xeb, 0x10
.byte 0xa1, 0x38, 0x0f, 0xa5, 0xb6, 0xa9, 0xb1, 0xbd, 0x1d, 0x34, 0xbc, 0x61, 0xcb, 0xfa, 0xe1, 0xad
.byte 0x93, 0x30, 0x3a, 0x8d, 0x48, 0x00, 0xb7, 0x03, 0x6e, 0x56, 0x0a, 0xb8, 0x1e, 0x67, 0x65, 0x68
.byte 0xf5, 0xfc, 0x06, 0x3d, 0x37, 0x7d, 0xec, 0x59, 0x12, 0xbb, 0xdf, 0xfb, 0x6f, 0xfe, 0x3b, 0x2f
.byte 0x5c, 0xd2, 0x50, 0xcf, 0x0d, 0xce, 0x09, 0xe9, 0xed, 0x21, 0xbf, 0x78, 0xd8, 0xcd, 0x28, 0x4a

.byte 0x87, 0xec, 0x1b, 0x2e, 0x78, 0x79, 0xb8, 0x4e, 0xd6, 0xdd, 0x49, 0x8b, 0x47, 0x35, 0xef, 0x55
.byte 0xa4, 0x4a, 0x92, 0xc6, 0xee, 0x6d, 0x75, 0x32, 0x46, 0x95, 0x83, 0xcd, 0x05, 0xfd, 0xbb, 0xf0
.byte 0x74, 0x1a, 0x68, 0xed, 0xdb, 0x2c, 0x3d, 0xa6, 0x6b, 0xe2, 0x12, 0x0b, 0x7d, 0x67, 0x9a, 0x2a
.byte 0xde, 0xfc, 0xb2, 0x6a, 0x48, 0xe5, 0x43, 0xa7, 0x7a, 0xfe, 0x7f, 0x52, 0xb1, 0x9b, 0x4c, 0x15
.byte 0x8d, 0xce, 0x37, 0x54, 0xea, 0x80, 0x97, 0xae, 0xe0, 0x1f, 0x3e, 0xff, 0xb7, 0xe4, 0xac, 0x64
.byte 0x4b, 0xe7, 0xb5, 0x1d, 0xf8, 0x28, 0x2d, 0xad, 0xbc, 0x2f, 0x93, 0x72, 0x63, 0x7b, 0xd1, 0xd9
.byte 0x5f, 0x91, 0x31, 0x20, 0x17, 0x8a, 0x62, 0x66, 0xc9, 0xf9, 0x13, 0xc1, 0xd7, 0xaf, 0xba, 0xf5
.byte 0x07, 0x19, 0x9f, 0xa5, 0x5d, 0xfb, 0x96, 0x89, 0xc4, 0x9e, 0x14, 0x3b, 0xbd, 0x5a, 0x7e, 0x26
.byte 0x25, 0x40, 0xa1, 0xc8, 0xdc, 0x77, 0xda, 0x10, 0xdf, 0xe9, 0xfa, 0x70, 0xa9, 0x53, 0xe8, 0x9d
.byte 0x4d, 0xca, 0x99, 0x38, 0xab, 0x3c, 0xb9, 0xbf, 0xf3, 0x1e, 0xa0, 0x22, 0x42, 0xa8, 0xd3, 0x0f
.byte 0xf1, 0x8c, 0xb6, 0x0a, 0x6c, 0x5b, 0xa3, 0xcf, 0x59, 0x08, 0x23, 0x1c, 0xc0, 0x76, 0xb0, 0xbe
.byte 0x29, 0xd2, 0x4f, 0x45, 0x00, 0x21, 0xe3, 0x0c, 0xb3, 0x7c, 0xa2, 0x50, 0xcb, 0x06, 0x61, 0x6e
.byte 0x94, 0xd8, 0xc3, 0xf2, 0x58, 0x85, 0x0d, 0x24, 0x84, 0x88, 0x90, 0x8f, 0x9c, 0x36, 0x01, 0x98
.byte 0x51, 0x5c, 0x5e, 0x27, 0x81, 0x33, 0x6f, 0x57, 0x3a, 0x8e, 0x39, 0x71, 0xb4, 0x03, 0x09, 0xaa
.byte 0x16, 0x02, 0xc7, 0x56, 0xc2, 0xe6, 0x82, 0x2b, 0x60, 0xd5, 0x44, 0x0e, 0x04, 0x3f, 0xc5, 0xcc
.byte 0x73, 0x11, 0xf4, 0xe1, 0x41, 0x86, 0x18, 0xd4, 0xd0, 0x30, 0xf7, 0x34, 0xf6, 0x69, 0xeb, 0x65

.byte 0xf1, 0x75, 0x5d, 0x70, 0x94, 0xbe, 0x1a, 0x43, 0xf3, 0xd1, 0x65, 0xbd, 0xea, 0x47, 0xa8, 0x4c
.byte 0xed, 0x64, 0x04, 0x1d, 0x68, 0x72, 0x25, 0x95, 0x15, 0x7b, 0xe2, 0x67, 0x23, 0xd4, 0xa9, 0x32
.byte 0x9a, 0x49, 0xc2, 0x8c, 0xf2, 0x0a, 0xff, 0xb4, 0x45, 0xab, 0xc9, 0x9d, 0x62, 0xe1, 0x3d, 0x7a
.byte 0xd2, 0xd9, 0x84, 0x46, 0x3a, 0x48, 0x5a, 0xe0, 0xe3, 0x88, 0x21, 0x14, 0x76, 0x77, 0x41, 0xb7
.byte 0x91, 0xcb, 0x34, 0x1b, 0x55, 0xb2, 0x29, 0x71, 0x16, 0x08, 0xaa, 0x90, 0xf4, 0x52, 0x86, 0x99
.byte 0xf6, 0xc6, 0xce, 0x1c, 0xa0, 0xd8, 0xfa, 0xb5, 0x9e, 0x50, 0x2f, 0x3e, 0x85, 0x18, 0x69, 0x6d
.byte 0x20, 0xb3, 0x7d, 0x9c, 0x74, 0x6c, 0xd6, 0xde, 0xe8, 0x44, 0x12, 0xba, 0x27, 0xf7, 0xa2, 0x22
.byte 0x10, 0xef, 0xf0, 0x31, 0xeb, 0xb8, 0x6b, 0xa3, 0xc1, 0x82, 0x5b, 0x38, 0x8f, 0xe5, 0xa1, 0x98
.byte 0x73, 0xbc, 0x5f, 0xad, 0x09, 0xc4, 0x61, 0x6e, 0xdd, 0x26, 0x4a, 0x40, 0x2e, 0x0f, 0x03, 0xec
.byte 0x07, 0x56, 0x13, 0x2c, 0x79, 0xcf, 0xb1, 0xbf, 0x83, 0xfe, 0x05, 0xb9, 0x54, 0x63, 0xc0, 0xac
.byte 0x11, 0xfc, 0x2d, 0xaf, 0xa7, 0x4d, 0x00, 0xdc, 0xc5, 0x42, 0x37, 0x96, 0x33, 0xa4, 0xb0, 0xb6
.byte 0xe6, 0xd0, 0x7f, 0xf5, 0x5c, 0xa6, 0x92, 0xe7, 0x4f, 0x2a, 0xc7, 0xae, 0x78, 0xd3, 0x1f, 0xd5
.byte 0x3f, 0xdf, 0x3b, 0xf8, 0x66, 0xf9, 0x6a, 0xe4, 0x1e, 0x7c, 0xee, 0xfb, 0x89, 0x4e, 0xdb, 0x17
.byte 0xda, 0x6f, 0x01, 0x4b, 0x30, 0x0b, 0xc3, 0xca, 0x0d, 0x19, 0x59, 0xc8, 0xe9, 0xcd, 0x24, 0x8d
.byte 0x81, 0x35, 0x7e, 0x36, 0x0c, 0xbb, 0xa5, 0x06, 0x53, 0x5e, 0x28, 0x51, 0x3c, 0x8e, 0x58, 0x60
.byte 0x87, 0x8b, 0x80, 0x9f, 0x39, 0x93, 0x97, 0x0e, 0xd7, 0x9b, 0xfd, 0xcc, 0x8a, 0x57, 0x2b, 0x02

.byte 0xb3, 0x57, 0xb8, 0x15, 0x42, 0x9a, 0x2e, 0x0c, 0xbc, 0xe5, 0x41, 0x6b, 0x8f, 0xa2, 0x8a, 0x0e
.byte 0xcd, 0x56, 0x2b, 0xdc, 0x98, 0x1d, 0x84, 0xea, 0x6a, 0xda, 0x8d, 0x97, 0xe2, 0xfb, 0x9b, 0x12
.byte 0x85, 0xc2, 0x1e, 0x9d, 0x62, 0x36, 0x54, 0xba, 0x4b, 0x00, 0xf5, 0x0d, 0x73, 0x3d, 0xb6, 0x65
.byte 0x48, 0xbe, 0x88, 0x89, 0xeb, 0xde, 0x77, 0x1c, 0x1f, 0xa5, 0xb7, 0xc5, 0xb9, 0x7b, 0x26, 0x2d
.byte 0x66, 0x79, 0xad, 0x0b, 0x6f, 0x55, 0xf7, 0xe9, 0x8e, 0xd6, 0x4d, 0xaa, 0xe4, 0xcb, 0x34, 0x6e
.byte 0x92, 0x96, 0xe7, 0x7a, 0xc1, 0xd0, 0xaf, 0x61, 0x4a, 0x05, 0x27, 0x5f, 0xe3, 0x31, 0x39, 0x09
.byte 0xdd, 0x5d, 0x08, 0xd8, 0x45, 0xed, 0xbb, 0x17, 0x21, 0x29, 0x93, 0x8b, 0x63, 0x82, 0x4c, 0xdf
.byte 0x67, 0x5e, 0x1a, 0x70, 0xc7, 0xa4, 0x7d, 0x3e, 0x5c, 0x94, 0x47, 0x14, 0xce, 0x0f, 0x10, 0xef
.byte 0x13, 0xfc, 0xf0, 0xd1, 0xbf, 0xb5, 0xd9, 0x22, 0x91, 0x9e, 0x3b, 0xf6, 0x52, 0xa0, 0x43, 0x8c
.byte 0x53, 0x3f, 0x9c, 0xab, 0x46, 0xfa, 0x01, 0x7c, 0x40, 0x4e, 0x30, 0x86, 0xd3, 0xec, 0xa9, 0xf8
.byte 0x49, 0x4f, 0x5b, 0xcc, 0x69, 0xc8, 0xbd, 0x3a, 0x23, 0xff, 0xb2, 0x58, 0x50, 0xd2, 0x03, 0xee
.byte 0x2a, 0xe0, 0x2c, 0x87, 0x51, 0x38, 0xd5, 0xb0, 0x18, 0x6d, 0x59, 0xa3, 0x0a, 0x80, 0x2f, 0x19
.byte 0xe8, 0x24, 0xb1, 0x76, 0x04, 0x11, 0x83, 0xe1, 0x1b, 0x95, 0x06, 0x99, 0x07, 0xc4, 0x20, 0xc0
.byte 0x72, 0xdb, 0x32, 0x16, 0x37, 0xa6, 0xe6, 0xf2, 0x35, 0x3c, 0xf4, 0xcf, 0xb4, 0xfe, 0x90, 0x25
.byte 0x9f, 0xa7, 0x71, 0xc3, 0xae, 0xd7, 0xa1, 0xac, 0xf9, 0x5a, 0x44, 0xf3, 0xc9, 0x81, 0xca, 0x7e
.byte 0xfd, 0xd4, 0xa8, 0x75, 0x33, 0x02, 0x64, 0x28, 0xf1, 0x68, 0x6c, 0xc6, 0x60, 0x7f, 0x74, 0x78

.global text
text:
.byte 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00

.global key
key:
.byte 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00

.global m
m:
.byte 0x03, 0x0c, 0x35, 0x3a, 0x50, 0x5f, 0x66, 0x69, 0x96, 0x99, 0xa0, 0xaf, 0xc5, 0xca, 0xf3, 0xfc

